.include "2313def.inc"

.equ	IND_DATA	=4		;IND_DATA  conn to pin #4
.equ	IND_CLOCK	=6		;IND_CLOCK conn to pin #6

.def	ACK		=R25		;(Global) Flag ACK "I2C device READY"

.def	DEL25		=R1		;(GL)DELAY 0.25 ms
.def	LOGO		=R2		;LOGO On/Off flag
.def	TC1		=R3		;TC1 Overflow counter

.def	BUTTON		=R4		;Buttons state register
.def	PNT		=R5		;Point position (CONSTANT SETTING)

;======================== *** SRAM SEGMENT ***==============================

	.DSEG  				;SRAM memory segment

BUFFER:		.byte	30	
POINTER:	.byte	1

DIGIT_CORR:	.byte	8
DIGIT_7SEG:	.byte	8		;DIGIT in 7-SEG Code 1
DIGIT_DEC:	.byte	8		;DIGIT in DEC Code 2
DIGIT_BIN:	.byte	4


DIGIT_DEC_M:	.byte	8		;DIGIT in DEC Code (9 - SIGN)
DIGIT_BIN_M:	.byte	4		;DIGIT of M Frequencity (4 - SIGN)


FRQ_BUF:	.byte	3

Count:		.byte	1

IND_COUNT:	.byte	1		;IND Counter
IND_ADR:	.byte	1		;IND Adress

ACKK:		.byte	1		;ACK subFLAG

;======================== *** CODE SEGMENT ***==============================

	.CSEG     	 		;begin of program memory
	.org 0

;-----------------SETUP INTERRUPTS HANDLERS---------------------------------

reset:	rjmp	start			; 00 Reset Handler
	reti				; 01 IRQ0 Handler
	reti				; 02 IRQ1 Handler
	reti				; 03 Timer1 Capture Handler
	reti				; 04 Timer1 Compare Handler
	rjmp	Overflow_TC1		; 05 Timer1 Overflow Handler
	rjmp	TimeKey			; 06 Timer0 Overflow Handler
	reti				; 07 UART RX Complete Handler
	reti				; 08 UDR Empty Handler
	reti				; 09 UART TX Complete Handler
	reti				; 0A Analog Comparator Handler
	
;--------------------- Messages & Constants --------------------------------

;    NAMES:    "0"  "1"  "2"  "3"  "4"  "5"  "6"  "7"  "8"  "9"  "A"  "B"  "C"  "D"  "E"  "F"	 "-"  " "

DIGITS:	.db	0xBB,0x09,0x73,0x5B,0xC9,0xDA,0xFA,0x19,0xFB,0xDB,0xF9,0xEA,0xB2,0x6B,0xF2,0xF0,0x40,0x00

;ORDER NUM:    "0"  "1"  "2"  "3"  "4"  "5"  "6"  "7"  "8"  "9"  "A"  "B"  "C"  "D"  "E"  "F"	 "10" "11"


;			     [10000000]            [1000000]            [100000]             [10000]               [1000]               [100]                [10]                  [1]

;(Bez SHUP)
DEC_CONST1:	.db	0x99,0x53,0x13,0x00, 0x0F,0x55,0x1A,0xCD, 0x01,0x88,0x82,0xAE, 0x00,0x27,0x40,0x45, 0x00,0x03,0xEC,0xD4, 0x00,0x00,0x64,0x7C, 0x00,0x00,0x0A,0x0C, 0x00,0x00,0x01,0x01

;(SHUP)
DEC_CONST2:	.db	0x95,0xBB,0x20,0x8E, 0x0E,0xF9,0x1C,0x2C, 0x01,0x7F,0x4F,0x9E, 0x00,0x26,0x54,0xC3, 0x00,0x03,0xD5,0x47, 0x00,0x00,0x62,0x21, 0x00,0x00,0x09,0xD0, 0x00,0x00,0x00,0xFB

;                     H                L   H                L   H                L    H               L    H              L     H               L    H              L     H              L  

;----------------------- BEGIN OF PROGRAMM ---------------------------------

;---------------------- *** setup part *** ---------------------------------

start:	ldi	r16,0xdf		;stack init SP=0xdf
	out	spl,r16			;

	sbi	ACSR,ACD		;Analog comparator off

	ldi	r16,0xFF		;Set Ini Ports func (DIRECTION)
	out	ddrb,r16		;	DDRB
	ldi	r16,0x00		;Set Ini Ports value (VALUE)
	out	portb,r16		;	PORTB

	ldi	r16,0b01010000		;Set Ini Ports func (DIRECTIO) (76543210)
	out	ddrd,r16		;	DDRD
	ldi	r16,0b00001100		;Set Ini Ports value (VALUE)   (76543210)
	out	portd,r16		;	PORTD

	ldi	r16,0b00000011
	out	TCCR0,r16		;T/C0 CK/64
	ldi	r16,0b00000010
	out	TIMSK,r16		;T/C0 Overflow INT enable
	
	ldi	r16,0b10000010
	out	TIMSK,r16		;T/C1 Overflow INT enable
	
;--------------------- Registers clearing ----------------------------------

	ldi	r31,0x00		;Load 0=>MSB (Z)
	ldi	r30,0x1d		;Load 29=>LSB (Z)
RgClr:	st	Z,R31			;0=>R(Z)
	dec	R30			;Counter=counter-1
	brne	RgClr			;Do until counter=0

;====================== Main program =======================================
		
	sei			;global INT enable	

;=========================== EEPROM READ  ==================================

	ldi	r16,8			;Counter = 8
	ldi	XL,(DIGIT_DEC_M)	;First adress of digit in 7SEG code

EER1:	out	EEAR,r16		;Ser ADDR in EEPROM
	sbi	EECR,EERE		;Read from EEPROM Enable
	in	r17,EEDR		;Read Data => Tmp

	st	X+,r17			;Store to RAM

	dec	r16
	brne	EER1

;=========================== LOGO DISPLAY ==================================
	
	clr	LOGO			;Clear LOGO flag
	
	ldi	r16,0x40		;
	sts	DIGIT_CORR,r16		;
	ldi	r16,0xF9		;
	sts	DIGIT_CORR+1,r16	;
	ldi	r16,0xFB		;
	sts	DIGIT_CORR+2,r16	;
	ldi	r16,0xBB		;
	sts	DIGIT_CORR+3,r16	;
	ldi	r16,0xDA		;
	sts	DIGIT_CORR+4,r16	;
	ldi	r16,0x0D		;
	sts	DIGIT_CORR+5,r16	;
	ldi	r16,0x5B		;
	sts	DIGIT_CORR+6,r16	;
	ldi	r16,0x40		;
	sts	DIGIT_CORR+7,r16	;Load "-8051.3-"
	
	rcall	DL25_255		;STOP
	
	ser	r16			;
	mov	LOGO,r16		;Set LOGO flag
	
;=========================== Main Part =====================================
	
	clr	TC1		;TC1=0
	sts	Count,TC1	;Count=0
	sts	ACKK,TC1	;ACKK=0

	ldi	r16,0x01
	sts	Pointer,r16	;Pointer=1

;===========================================================================

LOOP:		ldi	r16,0b00100000		;
		out	MCUCR,r16		;SLEEP enable
	
		sleep				;AT90S2313=>SLEEP
	
;================= KEY PRESS ==============================================

		sbrs	BUTTON,2		;
		rjmp	BUT1_0			;
		sbrs	BUTTON,3		;
		rjmp	BUT2_0			;
		rjmp	LOOP			;BUTTON Capture

;---------------------- BUTTON 1 ------------------------------------------

BUT1_0:		rcall	BUT_HALT		;STOP 

		clr	LOGO			;COUNTER OFF	
	
		rcall	DD_CLEAR		;CLEAR RAM DIGIT_DEC

		clr	PNT			;
		inc	PNT			;Set POINT position

		ldi	XH,high(DIGIT_DEC+7)	;
		ldi	XL,low(DIGIT_DEC+7)	;SET position

M1:		clr	r17			;Clear DIGIT counter

M2:		rcall	DEC_2_7SEG		;
		rcall	SEG7_CORR		;Digit`s restructure	

M3:		sbrs	BUTTON,2		;
		rjmp	BUT1_1			;
		sbrs	BUTTON,3		;
		rjmp	BUT2_1			;
		rjmp	M3			;Button CAPTURE

BUT1_1:		rcall	BUT_HALT		;STOP 

		inc	PNT			;Move POINT
		mov	r16,PNT			;
		cpi	r16,0x09		;
		breq	M7			;PNT=9? (END of programming)

		dec	XL			;DEC ADR DIGIT_DEC
		
		mov	r16,PNT			;
		cpi	r16,0x08		;
		brne	M6			;PNT=8? (for sign plasement)
		ldi	r17,0x10		;
		st	X,r17			;"-" ==> RAM
		rjmp	M2			;
		
M6:		st	X,XH			;ZERO ==> RAM
		
		rjmp	M1

BUT2_1:		rcall	BUT_HALT		;STOP 		
		
		mov	r16,PNT			;
		cpi	r16,0x08		;
		brne	M5			;PNT=8? (for sign plasement)
		
		inc	r17			;INC r17
		cpi	r17,0x12		;
		brne	M4			;
		ldi	r17,0x10		;If DIGIT=12 then DIGIT=10
		rjmp	M4

M5:		inc	r17			;INC DIGIT
		cpi	r17,0x0A		;
		brne	M4			;
		clr	r17			;If DIGIT>10 then DIGIT=0
		
M4:		st	X,r17			;Store DIGIT ==> RAM

		rjmp	M2

;------------- END of PROGRAMMING -----------------------------------------

M7:		ldi	XL,(DIGIT_DEC)		;First adress of digit in 7SEG code
		ldi	YL,(DIGIT_DEC_M)	;First adress of digit in 7SEG code (CONSTANT)

		ldi	r16,8			;Counter = 8
		
M8:		ld	r17,X+			;Load to r17 7SEG digit whith post adr INCR
		st	Y+,r17			;Store 7SEG ZERO Corrected digit whith post adr INCR 
		dec	r16			;
		brne	M8			;

;-------------- EEPROM WRITE  ---------------------------------------------


		ldi	r16,8			;Counter = 8
		ldi	XL,(DIGIT_DEC)		;First adress of digit in 7SEG code

EEW1:		sbic	EECR,EEWE		;WAIT for EEPROM READY
		rjmp	EEW1			;

		ld	r17,X+		

		out	EEDR,r17		;80h=>EEDATA
		out	EEAR,r16		;NP=>EEADRESS 
		cli				;Global INT Disable
		sbi	EECR,EEMWE		;MASTER WRITE ENABLE 
		sbi	EECR,EEWE		;WRITE to EEPROM ENABLE
		sei				;Global INT Enable

		dec	r16
		brne	EEW1

EEW2:		sbic	EECR,EEWE		;WAIT for EEPROM READY
		rjmp	EEW2			;

;--------------------------------------------------------------------------

		ser	r16			;
		mov	LOGO,r16		;COUNTER ON
		rjmp	LOOP			;JUMP LOOP

;---------------------- BUTTON 2 ------------------------------------------

BUT2_0:		rcall	BUT_HALT		;STOP 		

		clr	LOGO			;COUNTER OFF
		
		ldi	XL,(DIGIT_DEC_M)	;First adress of digit in 7SEG code
		ldi	YL,(DIGIT_DEC	)	;First adress of digit in 7SEG code (CONSTANT)

		ldi	r16,8			;Counter = 8
		
M9:		ld	r17,X+			;Load to r17 7SEG digit whith post adr INCR
		st	Y+,r17			;Store 7SEG ZERO Corrected digit whith post adr INCR 
		dec	r16			;
		brne	M9			;

		rcall	DEC_2_7SEG		;
		rcall	SEG7_CORR		;Digit`s restructure	

		rcall	DL25_255		;STOP

		ser	r16			;
		mov	LOGO,r16		;COUNTER ON
		rjmp	LOOP

;========================== END of MAIN ====================================

.include "timekey.asm"
.include "DEC27SEG.asm"
.include "BIN2DEC.asm"
.include "DEC2BIN.asm"
.include "ZERO.asm"
.include "TC1_OVER.asm"
.include "I2C.asm"




;==================  DELAY 025s ============================================

DL25:		cli
		push	r16
		ldi	r16,50
		mov	DEL25,r16
		pop	r16
		sei
DEL1:		tst	DEL25
		brne	DEL1

		RET	

;==================  DELAY 025s*255 ========================================

DL25_255:	ldi	r16,255			;
LG1:		dec	r16			;
		rcall	DL25			;
		tst	r16			;
		brne	LG1			;Delay DL25*255

		RET

;==================  BUTTON HALT   =========================================

BUT_HALT:	ldi	r16,55			;
BH1:		dec	r16			;
		rcall	DL25			;
		tst	r16			;
		brne	BH1			;Delay DL25*55		

		RET

;==================  CLEAR DIGIT_DEC RAM  ==================================

DD_CLEAR:	ldi	XH,high(DIGIT_DEC+1)	;
		ldi	XL,low(DIGIT_DEC+1)	;

		ldi	r16,0x10		;
		sts	DIGIT_DEC,r16		;Store "-" at first position

		ldi	r16,7			;
		clr	r17			;
DD1:		st	X+,r17			;
		dec	r16			;
		brne	DD1			;Clear DIGIT_DEC RAM

		RET

;==================     7SEG ===> CORR    ==================================

SEG7_CORR:	push	r16			;	
		push	XL			;
		push	YL			;
		push	r17			;Push Rx in STACK

		ldi	r16,8			;Counter=8
		ldi	XL,DIGIT_7SEG		;First adress of digit in 7SEG code
		ldi	YL,DIGIT_CORR		;First adress of digit in 7SEG ZERO Corrected code

S1:		ld	r17,X+			;Load to r17 7SEG digit whith post adr INCR
		st	Y+,r17			;Store 7SEG ZERO Corrected digit whith post adr INCR 
		dec	r16			;
		brne	S1			;
	
		pop	r17			;	
		pop	YL			;
		pop	XL			;
		pop	r16			;Pop Rx from STACK

		RET

;===========================================================================

